#include <compression/wah_compression.h>
#include <iostream>

using namespace aidbms::compression;
using namespace aidbms::benchmarks;


/**
 * @brief WAHCompression::compress
 * @param bit_stream
 * @return compressed bitstream
 */
std::vector<WAHBitVector> WAHCompression::compress(BitStream &bit_stream)
{
    std::vector<WAHBitVector> compressed;

    // iterate over bit stream in blocks of 31 Bits:
    //      if all values in a block are equal:
    //          set is_fill in WAHBitVector to 1 to indicate that we have a fill word here
    //          set count to 1
    //          set fill_bit (e.g. 0 if the block contains only zeros)
    //          while the following blocks is_compressible() and the elements are equals to the fill bit:
    //              count++
    //      else
    //          set is_fill in WAHBitVector to zero to indicate that the we have a literal word here
    //      append the WAHBitVector to the compressed bitstream

    return compressed;
}


/**
 * @brief WAHCompression::decompress
 * @param compressed
 * @return decompressed bitstream
 */
BitStream WAHCompression::decompress(std::vector<WAHBitVector> &compressed)
{
    BitStream stream;
    // iterate over the vector of WAHBitVectors:
    //      if bitvector.is_literal:
    //          iterate over the remaining bits and append them to the bitstream
    //      else:
    //          number_of_elements <- bitvector.count x 31
    //          append number_of_elements fill_bits to the bitstream

    return stream;
}


/**
 * @brief WAHCompression::is_stream_compressible
 * @param bit_stream
 * @param index
 * @return true if the string is compressible
 */
bool WAHCompression::is_stream_compressible(aidbms::benchmarks::BitStream &bit_stream, const std::size_t index) const
{
    if(index + 31 > bit_stream.size()) {
        return false;
    }

    const auto comparator = bit_stream[index];
    for(auto i = 1; i < 31; i++) {
        if(bit_stream[index + i] != comparator) {
            return false;
        }
    }

    return true;
}


/**
 * @brief WAHCompression::output_file
 * @param bitstream
 * @param filename
 */
void WAHCompression::output_file(aidbms::benchmarks::BitStream &bitstream, const std::string filename){
    std::ofstream outfile (filename, std::ios::out);
    for (auto bit : bitstream) {
            outfile << bit;
    }
    outfile.close();
}


/**
 * @brief WAHCompression::output_file
 * @param bitstream_vector
 * @param filename
 */
void WAHCompression::output_file(std::vector<WAHBitVector> &bitstream_vector, const std::string filename){
    std::ofstream outfile (filename, std::ios::out);
    for (auto i = 0u; i < bitstream_vector.size(); i++) {
        if (i > 0)
            outfile << std::endl;
        outfile << bitstream_vector[i].is_fill();
        for (int j = sizeof (WAHBitVector) * 8 - 2; j >= 0; j--) {
            outfile << bitstream_vector[i].get(j);
        }
    }
    outfile.close();
}
